CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
81 lines
2.4 KiB
C++
81 lines
2.4 KiB
C++
// Copyright (c) Advanced Micro Devices, Inc., or its affiliates.
|
|
// SPDX-License-Identifier: MIT
|
|
|
|
#ifndef CK_ARRAY_MULTI_INDEX_HPP
|
|
#define CK_ARRAY_MULTI_INDEX_HPP
|
|
|
|
#include "common_header.hpp"
|
|
|
|
namespace ck {
|
|
|
|
template <index_t N, typename T = index_t>
|
|
using MultiIndex = Array<T, N>;
|
|
|
|
template <typename IdxType = index_t, typename... Xs>
|
|
__host__ __device__ constexpr auto make_multi_index(Xs&&... xs)
|
|
{
|
|
return make_array<IdxType>(IdxType{xs}...);
|
|
}
|
|
|
|
template <index_t NSize, typename IdxType = index_t>
|
|
__host__ __device__ constexpr auto make_zero_multi_index()
|
|
{
|
|
return unpack([](auto... xs) { return make_multi_index<IdxType>(xs...); },
|
|
typename uniform_sequence_gen<NSize, 0>::type{});
|
|
}
|
|
|
|
template <typename T>
|
|
__host__ __device__ constexpr auto to_multi_index(const T& x)
|
|
{
|
|
return unpack([](auto... ys) { return make_multi_index(ys...); }, x);
|
|
}
|
|
|
|
template <index_t NSize, typename X>
|
|
__host__ __device__ constexpr auto operator+=(MultiIndex<NSize>& y, const X& x)
|
|
{
|
|
static_assert(X::Size() == NSize, "wrong! size not the same");
|
|
static_for<0, NSize, 1>{}([&](auto i) { y(i) += x[i]; });
|
|
return y;
|
|
}
|
|
|
|
template <index_t NSize, typename X>
|
|
__host__ __device__ constexpr auto operator-=(MultiIndex<NSize>& y, const X& x)
|
|
{
|
|
static_assert(X::Size() == NSize, "wrong! size not the same");
|
|
static_for<0, NSize, 1>{}([&](auto i) { y(i) -= x[i]; });
|
|
return y;
|
|
}
|
|
|
|
template <index_t NSize, typename T>
|
|
__host__ __device__ constexpr auto operator+(const MultiIndex<NSize>& a, const T& b)
|
|
{
|
|
using type = MultiIndex<NSize>;
|
|
static_assert(T::Size() == NSize, "wrong! size not the same");
|
|
type r;
|
|
static_for<0, NSize, 1>{}([&](auto i) { r(i) = a[i] + b[i]; });
|
|
return r;
|
|
}
|
|
|
|
template <index_t NSize, typename T>
|
|
__host__ __device__ constexpr auto operator-(const MultiIndex<NSize>& a, const T& b)
|
|
{
|
|
using type = MultiIndex<NSize>;
|
|
static_assert(T::Size() == NSize, "wrong! size not the same");
|
|
type r;
|
|
static_for<0, NSize, 1>{}([&](auto i) { r(i) = a[i] - b[i]; });
|
|
return r;
|
|
}
|
|
|
|
template <index_t NSize, typename T>
|
|
__host__ __device__ constexpr auto operator*(const MultiIndex<NSize>& a, const T& b)
|
|
{
|
|
using type = MultiIndex<NSize>;
|
|
static_assert(T::Size() == NSize, "wrong! size not the same");
|
|
type r;
|
|
static_for<0, NSize, 1>{}([&](auto i) { r(i) = a[i] * b[i]; });
|
|
return r;
|
|
}
|
|
|
|
} // namespace ck
|
|
#endif
|